from Paperl.Paperui.Widgets.widget import Widget
from typing import Literal

try:
    from tinui import TinUI


    class DWindow(Widget):
        def __init__(self, parent: Widget):
            self.build(parent.Me)

        def build(self, parent: Widget):
            self.Me = TinUI.TinUI(parent)

        def addButton(self, width: int = 100, height: int = 60, text: str = "Button", anchorType="nw",
                      font: tuple[str, int] = ("微软雅黑", 10), command=None,
                      foreground: str = "#000000", background: str = "#CCCCCC",
                      activeForeground: str = "#000000", activeBackground: str = "#999999"):
            me = self.Me.add_button(pos=(width, height), text=text, anchor=anchorType, font=font, command=command,
                                    bg=background, fg=foreground, activebg=activeBackground,
                                    activefg=activeForeground)
            return {"button": me[0], "id": me[4]}

        def addRoundButton(self, width: int = 100, height: int = 60, text: str = "Button", anchorType="nw",
                           font: tuple[str, int] = ("微软雅黑", 10), command=None,
                           foreground: str = "#1b1b1b", background: str = "#fbfbfb",
                           activeForeground: str = "#5d5d5d", activeBackground: str = "#f5f5f5"):
            me = self.Me.add_button2(pos=(width, height), text=text, anchor=anchorType, font=font, command=command,
                                     bg=background, fg=foreground, activebg=activeBackground,
                                     activefg=activeForeground)
            return {"button": me[0], "id": me[4]}

        def addTooltip(self, widget, foreground: str = "#3b3b3b", background: str = "#e7e7e7", borderColor="#e1e1e1",
                       text: str = "Button", delay: int = 0.05,
                       font: tuple[str, int] = ("微软雅黑", 10)):
            me = self.Me.add_tooltip(uid=widget["id"], bg=background, fg=foreground, font=font, text=text, delay=delay,
                                     outline=borderColor)
except:
    pass



from Paperl.Paperui.Widgets.window import Window


__all__ = ["example", "compile", "buildExample", "buildDocs"]


docs = """
'''
title :: 设置为窗口标题

theme ::
    mode :: 设置窗口主题 ::
                    light -> 浅色模式,
                    dark -> 深色模式
    style :: 设置主题 ::
                    aqua,
                    default,
                    clam,
                    classic,
                    sunValley,
                    vista
    style-attr :: 设置主题的额外属性 ::
                                light -> 浅色模式,
                                dark -> 深色模式,
                                auto -> 自动模式
    use-mica :: 是否启用Windows11云母特效 ::
                                    True -> 启用,
                                    False -> 禁用
    use-mica-attr :: 设置Windows11云母特效的主题 ::
                                            light -> 浅色模式,
                                            dark -> 深色模式,
                                            auto -> 自动模式

headerBar ::
    mode :: 设置标题栏的模式 ::
                    ex -> 仅Windows系统
    border :: 设置窗口是否保留边框 ::
                            True -> 启用,
                            False -> 禁用
    sizeGrip :: 设置窗口是否有大小调整手柄 ::
                                    True -> 启用,
                                    False -> 禁用
    use-sunValley :: 启用SunValley主题，并对其主题进行设置 ::
                                                True -> 启用,
                                                False -> 禁用
    use-sunValley-attr :: 设置SunValley的主题配置 ::
                                            light -> 浅色模式,
                                            dark -> 深色模式,
                                            auto -> 自动模式
'''
"""


example = """
title = "Paperl"

theme = {
    "mode": "light",
    "style": "default",
    "style-attr": "auto",
    "use-mica": False,
    "use-mica-attr": "auto"
}
"""


def compile(window: Window):
    buildExample()

    def checkTitle():
        try:
            window.setTitle(title)
        except:
            pass

    def checkStyle():
        try:
            mode = theme["mode"]
        except:
            mode = "light"
        try:
            style = theme["style"]
        except:
            from sys import platform
            if platform == "win32":
                style = "vista"
            else:
                style = "default"
        try:
            style_attr = theme["style-attr"]
        except:
            style_attr = "auto"
        try:
            use_mica = theme["use-mica"]
        except:
            use_mica = False
        try:
            use_mica_attr = theme["use-mica-attr"]
        except:
            use_mica_attr = "auto"

        if mode == "dark":
            window.setDarkTheme()
        else:
            window.setLightTheme()

        if use_mica:
            window.useMica(use_mica_attr)

        if style == "default":
            window.useStyleDefault()
        elif style == "sunValley":
            window.useStyleSunValley(style_attr)
        elif style == "classic":
            window.useStyleClassic()
        elif style == "clam":
            window.useStyleClam()
        elif style == "aqua":
            window.useStyleAqua()
        elif style == "vista":
            window.useStyleVista()
        else:
            from Paperl.Paperc import prWarring
            prWarring("Papers -> Compile -> This component was not found")

    def checkHeaderBar():
        try:
            mode = headerBar["mode"]
        except:
            from sys import platform
            if platform == "win32":
                mode = "ex"
            else:
                mode = ""
        if mode == "ex":
            try:
                border = headerBar["border"]
            except:
                border = True
            try:
                sizeGrip = headerBar["sizeGrip"]
            except:
                sizeGrip = False
            try:
                useSunValley = headerBar["use-sunValley"]
            except:
                useSunValley = False
            try:
                useSunValley_attr = headerBar["use-sunValley-attr"]
            except:
                useSunValley_attr = "auto"
            headerbar = window.createHeaderBarEx(border, sizeGrip)
            if useSunValley:
                headerbar.useSunValley(useSunValley_attr)
            headerbar.setTitle(window.getTitle())
        return headerbar

    try:
        from papers import title
    except:
        pass
    else:
        checkTitle()

    try:
        from papers import theme
    except:
        pass
    else:
        checkStyle()

    try:
        from papers import headerBar
    except:
        pass
    else:
        headerbar = checkHeaderBar()

    return {"headerBar": headerbar}


def buildDocs():
    from os.path import exists
    if not exists("papers-docs.py"):
        with open("papers-docs.py", "w+", encoding="gbk") as file:
            file.write(docs)
            file.close()


def buildExample():
    from os.path import exists
    if not exists("papers.py"):
        with open("papers.py", "w+", encoding="gbk") as file:
            file.write(example)
            file.close()



from Paperl.Paperui.Widgets.label import Label
from Paperl.Paperui.Widgets.image import Image
from Paperl.Paperui.Widgets.widget import Widget


class Photo(Label):
    def __init__(self, parent: Widget, image: Image):
        super().__init__(parent)
        self.setImage(image.export())



from Paperl.Paperc import prWarring
from Paperl.Paperui.Widgets.widget import Widget


class SysTray(object):
    def __init__(self, parent: Widget, tooltip: str = "托盘图标"):
        self.parent = parent
        self.build(parent.Me, tooltip)

    def build(self, parent, tooltip: str = "托盘图标"):
        try:
            from tkdev4.devtcl import DevTray
        except:
            prError("tkDev4 -> tkdev4 needs to be installed")
        else:
            try:
                self.Me = DevTray(parent, text=tooltip)
            except:
                prWarring("tkdev4 -> Parameter error")
            else:
                self.onProc(self.proc)

    def onProc(self, eventFunc):
        self.Me.menu_func = eventFunc

    def proc(self, event, iconId, icon, x, y):
        """
        内置菜单，用于初始化。
        """
        if event == "WM_RBUTTONDOWN":
            from Paperl.Paperui.Widgets.menu import Menu
            menu = Menu()
            menu.setBorder(0)
            menu.setTearOff(False)
            menu.addCommand(label="退出", command=self.parent.destroy)
            menu.popup(x, y)

    def add(self):
        self.Me.taskbar_add()

    def delete(self):
        self.Me.taskbar_delete()

    def deleteMe(self):
        self.Me.taskbar_delete_me()



from Paperl.Paperui.Drawing.DWindow import DWindow
from Paperl.Paperui.Widgets.widget import Widget


try:
    from tinui import TinUI

    class DTheme:
        def __init__(self, parent: DWindow):
            self.build(parent.Me)

        def build(self, parent: Widget):
            self.Me = TinUI.TinUITheme(parent)

        def setAuto(self):
            self.Me.change_theme_name("auto")

        def setLight(self):
            self.Me.change_theme_name("light")

except:
    pass